package com.mh.user.serialport;

import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.NowDataService;
import com.mh.user.utils.AnalysisReceiveOrder485;
import com.mh.user.utils.SendOrderUtils;
import com.mh.user.utils.SpringBeanUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import purejavacomm.SerialPort;

/**
 * @author nxr
 * @title ：
 * @description ： 串口发送和接收处理,操作类
 * @updateTime 2022-08-10
 * @throws ：
 */
@Slf4j
public class SerialPortSingle2 {

    private  String receiveStr = null;
    private  int baudrate=9600;
    private String parity=null;

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();
    DeviceInstallService deviceInstallService = context.getBean(DeviceInstallService.class);
    NowDataService nowDataService = context.getBean(NowDataService.class);
    BuildingService buildingService = context.getBean(BuildingService.class);
    public String serialPortSend(DeviceCodeParamEntity deviceCodeParamEntity){
        SerialPort serialPort = null;
        String rtData="";
        String comName=deviceCodeParamEntity.getDataCom().toUpperCase();
        //if (port.contains(comName)){
            try{
                try{
                    baudrate=deviceCodeParamEntity.getBaudrate();
                    parity=deviceCodeParamEntity.getParity();
                    if (parity==null || parity.equals("") || parity.equalsIgnoreCase("none")){
                        serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
                    }else{
                        serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
                    }
                    //向串口发送指令
                    log.info("-----------------------------单抄向串口"+serialPort+"发送指令！-----------------------------");
                    if (serialPort != null) {
                        String sendStr=SendOrderUtils.getSendStr(deviceCodeParamEntity);
                        SerialTool.sendToPort(SerialTool.HexString2Bytes(sendStr), serialPort,sendStr,deviceCodeParamEntity.getDeviceType());
                        Thread.sleep(1500);
                    }
                }catch(Exception e){
                   e.printStackTrace();
                }
                //对返回数据进行相关解析处理
                receiveStr=null;
                byte[] bytes = SerialTool.readFromPort(serialPort); //读取串口数据
                try {
                    String byteStr = new String(bytes, 0, bytes.length).trim();
                    receiveStr = receiveStr + printHexString(bytes);
                    //去掉空格和null
                    receiveStr = receiveStr.replace("null", "");
                    receiveStr = receiveStr.replace(" ", "");
                    log.info("串口"+serialPort+"接收数据：" + receiveStr + "，大小： " + receiveStr.length());
                } catch (NullPointerException e) {
                    SerialTool.closePort(serialPort);
                    log.info("单抄串口"+serialPort+"异常,没有数据返回！关闭串口");
                }
                //返回值全部变成大写
                String receiveData = receiveStr.toUpperCase();
                //截取去掉FE
                String dataStr = receiveData.replace("FE", "");
                String deviceType=deviceCodeParamEntity.getDeviceType();
                String deviceAddr=deviceCodeParamEntity.getDeviceAddr();
                String registerAddr=deviceCodeParamEntity.getRegisterAddr();
                String brand=deviceCodeParamEntity.getBrand();
                String buildingId=deviceCodeParamEntity.getBuildingId();
                String buildingName=buildingService.queryBuildingName(buildingId); //查询楼栋名称

                deviceInstallService.updateOnline(deviceAddr,deviceType,buildingId,"在线"); //设备在线
                log.info(deviceType+"在线,设备号："+deviceAddr+",所属楼栋："+buildingName);
                if (deviceType.equals("热泵")){
                    String strState=nowDataService.selectState(buildingId,deviceAddr);
                    if (strState!=null && strState.equals("离线")){ //采集到数据
                        nowDataService.updateRunState(buildingId,deviceAddr,"不运行"); //监控界面状态表热泵在线状态
                    }
                }
                try{
                    if ((dataStr.length() == 18 || dataStr.length() == 70 || dataStr.length() == 44)  && deviceType.equals("水表")) {
                        rtData=analysisReceiveOrder485.analysisWtMeterOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if ((dataStr.length() == 36 || dataStr.length() == 44 || dataStr.length()==40 || dataStr.length()==50) &&  deviceType.equals("电表")) {
                        rtData=analysisReceiveOrder485.analysisMeterOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("压变")) {
                        rtData=analysisReceiveOrder485.analysisPressureOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if ((dataStr.length() == 30) && deviceType.equals("状态检测")) {//五路状态读取,兼容旧版系统
                        analysisReceiveOrder485.analysisStateOrder485(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("水位开关") && (registerAddr.equals("0018") || registerAddr.equals("0017"))) {
                        rtData=analysisReceiveOrder485.analysisRelayOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("热泵")) {
                        rtData=analysisReceiveOrder485.analysisPumpOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("时控")) {
                        rtData=analysisReceiveOrder485.analysisTimeSetOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("水位开关") && registerAddr.equals("0010")){ //热泵状态
                        rtData=analysisReceiveOrder485.analysisPumpStateOrder2(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("温度变送器")) {
                        rtData=analysisReceiveOrder485.analysisMulTempOrder4852(dataStr,registerAddr,brand,buildingId);
                    } else if (deviceType.equals("热泵状态")){
                        rtData=analysisReceiveOrder485.analysisPumpStateOrder2(dataStr,registerAddr,brand,buildingId);
                    }
                }catch (Exception e){
                    log.error(deviceCodeParamEntity.getDeviceType()+"单抄保存数据库失败！");
                    SerialTool.closePort(serialPort);
                }
                SerialTool.closePort(serialPort);
                System.out.println("关闭"+serialPort);
                Thread.sleep(500);
                log.info("-----------------------------"+serialPort+"单抄结束！-----------------------------");
                return rtData;
            }catch (Exception e){
//                e.printStackTrace();
            }
        //}else {
            //log.info("串口："+comName+"不存在!");
        //}
        return "fail";
    }
    /**
     * 字节数组转16进制字符串
     * @param b 字节数组
     * @return 16进制字符串
     */
    public static String printHexString(byte[] b) {
        StringBuilder sbf = new StringBuilder();
        for (byte value : b) {
            String hex = Integer.toHexString(value & 0xFF);
            if (hex.length() == 1) {
                hex = '0' + hex;
            }
            sbf.append(hex.toUpperCase()).append(" ");
        }
        return sbf.toString().trim();
    }

    /**
     * 十六进制字符串转byte[]
     * @param hex 十六进制字符串
     * @return byte[]
     */
    public static byte[] hexStr2Byte(String hex) {
        if (hex == null) {
            return new byte[] {};
        }

        // 奇数位补0
        if (hex.length() % 2 != 0) {
            hex = "0" + hex;
        }

        int length = hex.length();
        ByteBuffer buffer = ByteBuffer.allocate(length / 2);
        for (int i = 0; i < length; i++) {
            String hexStr = hex.charAt(i) + "";
            i++;
            hexStr += hex.charAt(i);
            byte b = (byte) Integer.parseInt(hexStr, 16);
            buffer.put(b);
        }
        return buffer.array();
    }
    /**
     * 16进制转换成为string类型字符串
     * @param s 待转换字符串
     */
    public static String hexStringToString(String s) {
        if (s == null || "".equals(s)) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, StandardCharsets.UTF_8);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return s;
    }
}
