package com.mh.user.service.impl;

import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.entity.DeviceInstallTempEntity;
import com.mh.user.entity.UploadDeviceInstallEntity;
import com.mh.user.exception.CommonNum;
import com.mh.user.exception.ServiceRuntimeException;
import com.mh.user.mapper.DeviceInstallMapper;
import com.mh.user.mapper.provider.DeviceInstallProvider;
import com.mh.user.model.DeviceModel;
import com.mh.user.serialport.SerialPortSendReceive;
import com.mh.user.service.DeviceInstallService;
import lombok.extern.slf4j.Slf4j;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.SelectProvider;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class DeviceInstallServiceImpl implements DeviceInstallService {

    @Autowired
    private DeviceInstallMapper deviceInstallMapper;

    @Override
    public int saveDevice(DeviceInstallEntity deviceInstallEntity) {
        return deviceInstallMapper.saveDevice(deviceInstallEntity);
    }

    @Override
    public int updateDevice(DeviceInstallEntity deviceInstallEntity) {
        return deviceInstallMapper.updateDevice(deviceInstallEntity);
    }

    @Override
    public void updateLastValue(Long id, String lastValue, Date lastDate) {
        deviceInstallMapper.updateLastValue(id,lastValue,lastDate);
    }

    @Override
    public List<DeviceInstallEntity> getAllDevice(int page, int limit) {
        return deviceInstallMapper.getAllDevice(page,limit);
    }

    @Override
    public List<DeviceInstallEntity> selectDeviceParams() {
        return deviceInstallMapper.selectDeviceParams();
    }

    @Override
    public List<DeviceInstallEntity> selectDeviceParams2() {
        return deviceInstallMapper.selectDeviceParams2();
    }

    @Override
    public List<DeviceInstallEntity> selectDevicePump() {
        return deviceInstallMapper.selectDevicePump();
    }

    // 查询所有启用的设备
    @Override
    public int getAllCount(){
        return deviceInstallMapper.getAllCount();
    }

    @Override
    public List<DeviceInstallEntity> queryDevice(String buildingId,String deviceType, String startDate, String endDate,String isOnline, String isUse,String isFault,int page, int limit) {
        return deviceInstallMapper.queryDevice(buildingId,deviceType,startDate, endDate,isOnline,isUse, isFault,page, limit);
    }

    @Override
    public int getCount(String buildingId,String deviceType, String startDate, String endDate,String isOnline, String isUse, String isFault, int page, int limit) {
        return deviceInstallMapper.getCount(buildingId,deviceType, startDate, endDate,isOnline,isUse, isFault, page, limit);
    }

    //查询设备故障情况
    @Override
    public int getIsFaultCount(String isFault,String deviceType){
        return deviceInstallMapper.getIsFaultCount(isFault,deviceType);
    }

    //查询设备在线情况
    @Override
    public int getIsOnlineCount(String isOnline,String deviceType){
        return deviceInstallMapper.getIsOnlineCount(isOnline,deviceType);
    }

    @Override
    public void updateOnline(String deviceAddr, String deviceType,String buildingId,String isOnline) {
        deviceInstallMapper.updateOnline(deviceAddr,deviceType,buildingId,isOnline);
    }

    @Override
    public void updateNotOnline(String deviceAddr, String deviceType,String buildingId,String isOnline) {
        deviceInstallMapper.updateNotOnline(deviceAddr,deviceType,buildingId,isOnline);
    }

    //根据通讯地址和设备类型查询对应的设备信息
    @Override
    public DeviceInstallEntity selectDevice(String deviceAddr, String deviceType,String buildingId) {
        return deviceInstallMapper.selectDevice(deviceAddr,deviceType,buildingId);
    }

    @Override
    public int selectDeviceCount(String deviceAddr, String deviceType) {
        return deviceInstallMapper.selectDeviceCount(deviceAddr,deviceType);
    }

    @Override
    public int deleteDevice(List<DeviceInstallEntity> records) {
        for(DeviceInstallEntity record:records) {
            deviceInstallMapper.deleteDevice(record.getId().toString());
        }
        return 0;
    }

    @Override
    public int deleteDevice(String id) {
        return deviceInstallMapper.deleteDevice(id);
    }

    //Excel导入相关处理函数
    @Override
    public void deleteDevice_install_temp() {
        deviceInstallMapper.deleteDevice_install_temp();
    }

    @Override
    public void insertDevice_install_temp(UploadDeviceInstallEntity uploadDeviceInstallEntity) {
        String deviceAddr = uploadDeviceInstallEntity.getDeviceAddr();
        String deviceName   = uploadDeviceInstallEntity.getDeviceName();
        String deviceType  = uploadDeviceInstallEntity.getDeviceType();
        int baudRate     = uploadDeviceInstallEntity.getBaudRate();
        String dataCom    = uploadDeviceInstallEntity.getDataCom();
        double ratio    = uploadDeviceInstallEntity.getRatio();
        String buildingId    = uploadDeviceInstallEntity.getBuildingId();
        String rowId         = uploadDeviceInstallEntity.getRowId();
        deviceInstallMapper.insertDevice_install_temp(deviceAddr, deviceName, deviceType, baudRate, dataCom, ratio, buildingId, rowId);
    }

    // 查询Excel导入的数据
    @Override
    public List<DeviceInstallTempEntity> queryExcelDevices(){
        return deviceInstallMapper.queryExcelDevices();
    };

    // 查询Excel导入的数据的记录数
    @Override
    public int queryExcelDevicesCount(){
        return deviceInstallMapper.queryExcelDevicesCount();
    };

    // 在导入中的数据有重复
    @Override
    public void updateDevice_install_temp__multiple() {
        deviceInstallMapper.updateDevice_install_temp__multiple();
    }

    /**
     * @author nxr
     * @title ：
     * @description ：判断导入资料数据的合法性
     * @updateTime 2022-06-19
     * @throws ：
     */
    public void updateDevice_install_temp(){
        //判断通讯地址，在导入中的数据有重复
        deviceInstallMapper.updateDevice_install_temp__multiple();
    }

    //从临时表查询插入
    @Override
    public void insertFromDevice_install_temp() {
        deviceInstallMapper.insertFromDevice_install_temp();
        deviceInstallMapper.deleteDevice_install_temp();
    }

    //修改设备启用状态
    @Override
    public void updateDeviceIsUse(String isUse, String deviceAddr) {
        deviceInstallMapper.updateDeviceIsUse(isUse,deviceAddr);
    }

    @Override
    public List<DeviceModel> selectDevices(String buildingId, String deviceType) {
        return deviceInstallMapper.selectDevices(buildingId, deviceType);
    }

    @Override
    public void updateDeviceFault(String isFault, String deviceAddr, String deviceType) {
        deviceInstallMapper.updateDeviceFault(isFault,deviceAddr,deviceType);
    }

    //查询设备品牌
    @Override
    public String selectBrand(String buildingId,String deviceAddr) {
        return deviceInstallMapper.selectBrand(buildingId,deviceAddr);
    }

    @Override
    public void updateIsUse(String isUse, String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.updateIsUse(isUse, deviceAddr, deviceType, buildingId);
    }

    @Override
    public void updateIsUse2(String isUse, String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.updateIsUse2(isUse, deviceAddr, deviceType, buildingId);
    }

    @Override
    public void updateIsUse3(String isUse, String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.updateIsUse3(isUse, deviceAddr, deviceType, buildingId);
    }

    @Override
    public void deletePump(String deviceAddr, String deviceType, String buildingId) {
        deviceInstallMapper.deletePump(deviceAddr, deviceType, buildingId);
    }

    @Override
    public void updateInitValue(String deviceAddr, String deviceType, String buildingId, String initValue) {
        deviceInstallMapper.updateInitValue(deviceAddr, deviceType, buildingId, initValue);
    }

    @Override
    public String selectLastDate(String deviceType, String deviceAddr,String buildingId) {
        return deviceInstallMapper.selectLastDate(deviceType,deviceAddr,buildingId);
    }

    @Override
    public int judgePump(String buildingId) {
        return deviceInstallMapper.judgePump(buildingId);
    }

    @Override
    public String selectSeat(String deviceType, String deviceAddr, String buildingId) {
        return deviceInstallMapper.selectSeat(deviceType, deviceAddr, buildingId);
    }
}

