package com.mh.user.utils;

import com.mh.user.entity.ChillersEntity;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.DeviceManageEntity;
import com.mh.user.model.OrderParamModel;
import com.mh.user.serialport.SerialPortListener;
import com.mh.user.serialport.SerialPortUtil;
import com.mh.user.serialport.SerialTool;
//import purejavacomm.SerialPort;
import gnu.io.SerialPort;
import io.netty.buffer.ByteBuf;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;

/**
 * @author ljf
 * @title ：
 * @description ： 发送指令工具类
 * @updateTime 2021-01-26
 * @throws ：
 */
@Slf4j
public class SendOrderUtils {

	/**
	 * 发送采集冷水机组
	 * @param chillersEntity
	 * @param ctx
	 * @param num
	 * @param size
	 */
	public static void sendChillersOrder(ChillersEntity chillersEntity, ChannelHandlerContext ctx, int num, int size) {
		// 继续发送下一个采集冷水机设备指令
		GetReadOrder485 getReadOrder485 = new GetReadOrder485();
		String sendStr = getReadOrder485.createChillersOrder(chillersEntity);
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 2.发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("客户端再次往服务端发送数据" + num + " 数据条数：" + size);
	}

	public static void sendMeterOrCloud(DeviceManageEntity deviceManageEntity,
										int num,
										String IP,
										String port,
										ChannelHandlerContext ctx) {
		if (deviceManageEntity.getParamId() == 1) {
			sendMeterOrder(deviceManageEntity, num, IP, port, ctx);
		} else if (deviceManageEntity.getParamId() == 3) {
			sendCloudOrder(deviceManageEntity, num, IP, port, ctx);
		}
	}
	/**
	 * 发送采集电表
	 * @param deviceManageEntity
	 * @param num
	 * @param IP
	 * @param port
	 * @param ctx
	 */
	public static void sendMeterOrder(DeviceManageEntity deviceManageEntity,
									  int num,
									  String IP,
									  String port,
									  ChannelHandlerContext ctx) {
//		String collectionNum = deviceManageEntity.getCollectionNum();
//		String sendStr = GetReadOrder485.createMeterOrder();
		String sendStr="";
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 2.发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("客户端再次往服务端发送数据" + num);
	}

	/**
	 * 发送采集冷量计工具类
	 * @param deviceManageEntity
	 * @param num
	 * @param IP
	 * @param port
	 * @param ctx
	 */
	public static void sendCloudOrder(DeviceManageEntity deviceManageEntity,
									  int num,
									  String IP,
									  String port,
									  ChannelHandlerContext ctx) {
		// 1.创建将要写出的数据
		String collectionNum = deviceManageEntity.getCollectionNum();
		String sendStr = GetReadOrder485.createCloudOrder(IP, port,
				deviceManageEntity.getDataCom(),
				collectionNum, "34");
//            String sendStr = "5803004900021914";
		ByteBuf buffer = getByteBuf(ctx, sendStr);
		// 2.发送数据
		ctx.channel().writeAndFlush(buffer);
		log.info("客户端再次往服务端发送数据" + sendStr);
	}

	//向串口发指令数据
	public static void sendSerialPort(DeviceCodeParamEntity deviceCodeParamEntity,SerialPort serialPort ) {
		//创建将要写出的数据
		OrderParamModel orderParamModel=new OrderParamModel();
		orderParamModel.setDeviceAddr(deviceCodeParamEntity.getDeviceAddr());//通讯地址
		orderParamModel.setDeviceType(deviceCodeParamEntity.getDeviceType());//设备类型
		orderParamModel.setBaudrate(deviceCodeParamEntity.getBaudrate());//波特率
		orderParamModel.setBrand(deviceCodeParamEntity.getBrand());//品牌
		orderParamModel.setFunCode(deviceCodeParamEntity.getFunCode());//功能码
		orderParamModel.setRegisterAddr(deviceCodeParamEntity.getRegisterAddr());//寄存器地址
		orderParamModel.setComName(deviceCodeParamEntity.getDataCom()); //串口
		orderParamModel.setDataValue(deviceCodeParamEntity.getDataValue());//传入数据

		String deviceType=deviceCodeParamEntity.getDeviceType();
		String registerAddr=deviceCodeParamEntity.getRegisterAddr();
		String sendStr=null;
		if (deviceType.equals("电表")){
			sendStr = GetReadOrder485.createMeterOrder(orderParamModel);
		}else if (deviceType.equals("水表")){
			sendStr = GetReadOrder485.createWtMeterOrder(orderParamModel);
		}else if (deviceType.equals("压变")){
			sendStr = GetReadOrder485.createPressureOrder(orderParamModel);
		}else if (deviceType.equals("热泵")){
			sendStr = GetReadOrder485.createPumpOrder(orderParamModel);
		}else if (deviceType.equals("温控")){
			sendStr = GetReadOrder485.createTempOrder(orderParamModel);
		}else if (deviceType.equals("时控")){
			sendStr = GetReadOrder485.createTimeSetOrder(orderParamModel);
		}else if (deviceType.equals("水位开关")){
			sendStr = GetReadOrder485.createRelayOrder(orderParamModel);
		}else if (deviceType.equals("状态检测")){
			sendStr = GetReadOrder485.createStateOrder(orderParamModel);
		}else if (deviceType.equals("温度变送器")){
			sendStr = GetReadOrder485.createMulTempOrder(orderParamModel);
		}else if (deviceType.equals("热泵状态")){
			sendStr = GetReadOrder485.createPumpStateOrder(orderParamModel);
		}

		String str=sendStr;
        try{
        	byte[] HEX = SerialPortListener.hexStr2Byte(str);
		    SerialPortUtil serialPortUtil = SerialPortUtil.getSerialPortUtil();
		    serialPortUtil.sendToPort(serialPort, HEX);

//			SerialTool serialPortUtil = SerialTool.getSerialPortUtil();
//			serialPortUtil.sendToPort(HEX,serialPort);
			log.info(deviceType+"向串口"+serialPort+"发送数据>>>>" + str);
		}catch (Exception e){
        	e.printStackTrace();
			log.error("向串口"+serialPort+"发送数据异常！");
		}
	}

	//向串口发指令数据
	public static String getSendStr(DeviceCodeParamEntity deviceCodeParamEntity) {
		//创建将要写出的数据
		OrderParamModel orderParamModel=new OrderParamModel();
		orderParamModel.setDeviceAddr(deviceCodeParamEntity.getDeviceAddr());//通讯地址
		orderParamModel.setDeviceType(deviceCodeParamEntity.getDeviceType());//设备类型
		orderParamModel.setBaudrate(deviceCodeParamEntity.getBaudrate());//波特率
		orderParamModel.setBrand(deviceCodeParamEntity.getBrand());//品牌
		orderParamModel.setFunCode(deviceCodeParamEntity.getFunCode());//功能码
		orderParamModel.setRegisterAddr(deviceCodeParamEntity.getRegisterAddr());//寄存器地址
		orderParamModel.setComName(deviceCodeParamEntity.getDataCom()); //串口
		orderParamModel.setDataValue(deviceCodeParamEntity.getDataValue());//传入数据

		String deviceType=deviceCodeParamEntity.getDeviceType();
		String registerAddr=deviceCodeParamEntity.getRegisterAddr();
		String sendStr=null;
		if (deviceType.equals("电表")){
			sendStr = GetReadOrder485.createMeterOrder(orderParamModel);
		}else if (deviceType.equals("水表")){
			sendStr = GetReadOrder485.createWtMeterOrder(orderParamModel);
		}else if (deviceType.equals("压变")){
			sendStr = GetReadOrder485.createPressureOrder(orderParamModel);
		}else if (deviceType.equals("热泵")){
			sendStr = GetReadOrder485.createPumpOrder(orderParamModel);
		}else if (deviceType.equals("温控")){
			sendStr = GetReadOrder485.createTempOrder(orderParamModel);
		}else if (deviceType.equals("时控")){
			sendStr = GetReadOrder485.createTimeSetOrder(orderParamModel);
		}else if (deviceType.equals("水位开关")){
			sendStr = GetReadOrder485.createRelayOrder(orderParamModel);
		}else if (deviceType.equals("状态检测")){
			sendStr = GetReadOrder485.createStateOrder(orderParamModel);
		}else if (deviceType.equals("温度变送器")){
			sendStr = GetReadOrder485.createMulTempOrder(orderParamModel);
		}else if (deviceType.equals("热泵状态")){
			sendStr = GetReadOrder485.createPumpStateOrder(orderParamModel);
		}

		return sendStr;
	}



	private static ByteBuf getByteBuf(ChannelHandlerContext ctx, String sendStr) {
		// 申请一个数据结构存储信息
		ByteBuf buffer = ctx.alloc().buffer();
		// 将信息放入数据结构中
		buffer.writeBytes(ExchangeStringUtil.hexStrToBinaryStr(sendStr));//对接需要16进制
		return buffer;
	}
}
